@@ -1,33 +1,23 @@
 #  File ergm/R/print.ergm.R
-#  Part of the statnet package, http://statnetproject.org
+#  Part of the statnet package, http://statnet.org
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) in
-#    http://statnetproject.org/attribution
+#    http://statnet.org/attribution
 #
-#  Copyright 2011 the statnet development team
+#  Copyright 2012 the statnet development team
 ######################################################################
 ###############################################################################
 # The <print.ergm> function prints summary information for a given ergm
-#
-# --PARAMETERS--
-#   x     :  an ergm object
-#   digits:  the number of significant digits for the coefficients;
-#            default=max(3, getOption("digits")-3)
-#   ...   :  additional parameters passed from within; these will be ignored
-#
-# --RETURNED--
-#   x
-# 
 ###############################################################################
 
 print.ergm <- function (x, digits = max(3, getOption("digits") - 3), ...) {
 #  if(!is.latent(x) || is.null(x$Z.mle)) {                             
   if(is.null(x$Z.mle)) {
    if(is.matrix(x$sample)){
-    if(!is.matrix(x$thetasample) && !is.null(x$iterations)){
-     cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
-    }
+#    if(!is.matrix(x$thetasample) && !is.null(x$iterations)){
+#     cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
+#    }
     cat("MCMC sample of size", x$samplesize, "based on: \n")
     print.default(format(x$MCMCtheta, digits = digits), print.gap = 2, 
         quote = FALSE)
@@ -35,9 +25,9 @@ print.ergm <- function (x, digits = max(3, getOption("digits") - 3), ...) {
     print.default(format(x$coef, digits = digits), print.gap = 2, 
         quote = FALSE)
    }else{
-    if (!is.null(x$iterations)) {
-      cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
-    }
+#    if (!is.null(x$iterations)) {
+#      cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
+#    }
     if(!all(x$theta1$independent)){
      cat("\nMPLE Coefficients:\n")
      print.default(format(x$coef, digits = digits), print.gap = 2, 
@@ -50,9 +40,9 @@ print.ergm <- function (x, digits = max(3, getOption("digits") - 3), ...) {
    }
   }
   else {
-    if (!is.null(x$iterations)) {
-      cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
-    }
+#    if (!is.null(x$iterations)) {
+#      cat("Newton-Raphson iterations: ", x$iterations[1], "\n")
+#    }
     cat("MLE and Posterior Mean of Coefficients:\n")
     temp <- cbind(x$MCMCtheta,x$coef)
     colnames(temp) <- c("MLE","MAP")
