@@ -82,7 +82,7 @@ void node_geodesics (int *edgelist, int *nnodes, int *nodelist,
   Q[Qtop++]=*source;  /* Push source onto top of queue */
   while (Qbottom<Qtop) {  /* Repeat until queue is empty */
     u=Q[Qbottom++]; /* Pop vertex off bottom of queue (it must be NONWHITE) */
-    for (j=2*nodelist[u-1]; edgelist[j]==u && j<twoe; j+=2) {
+    for (j=2*nodelist[u-1]; j<twoe && edgelist[j]==u; j+=2) {
       v=edgelist[j+1];
       if (nodecolor[v-1]==0) { /* WHITE */
         nodecolor[v-1]=1; /* NONWHITE */
