@@ -1,12 +1,12 @@
 /*
  *  File ergm/src/wtedgetree.h
- *  Part of the statnet package, http://statnetproject.org
+ *  Part of the statnet package, http://statnet.org
  *
  *  This software is distributed under the GPL-3 license.  It is free,
  *  open source, and has the attribution requirements (GPL Section 7) in
- *    http://statnetproject.org/attribution
+ *    http://statnet.org/attribution
  *
- *  Copyright 2011 the statnet development team
+ *  Copyright 2012 the statnet development team
  */
 #ifndef WTEDGETREE_H
 #define WTEDGETREE_H
@@ -17,8 +17,6 @@
 #include <Rmath.h>
 #include "edgetree.h"
 
-
-
 /* WtTreeNode is just like TreeNode but with an extra field for a
    weight, or value, that might be associated with the node */
 typedef struct WtTreeNodestruct {
@@ -47,7 +45,7 @@ typedef struct WtTreeNodestruct {
      the smallest index of an edge object not being used.  
    outdegree[] and indegree[] are continually updated to give
      the appropriate degree values for each vertex.  These should
-     point to Vertex-vectors of length nnodes.  
+     point to Vertex-vectors of length nnodes+1.  
    value:  optional value(s) associated with this network 
    Dur_Inf:  See typedef for Dur_Infstruct above
 */
@@ -64,7 +62,7 @@ typedef struct WtNetworkstruct {
   Edge next_outedge;
   Vertex *indegree;
   Vertex *outdegree;
-  double *value;
+  double *value;  
   Dur_Inf duration_info;
   Edge maxedges;
 } WtNetwork;
@@ -72,19 +70,30 @@ typedef struct WtNetworkstruct {
 /* Initialization and destruction. */
 WtNetwork WtNetworkInitialize(Vertex *tails, Vertex *heads, double *weights, Edge nedges,
 			      Vertex nnodes, int directed_flag, Vertex bipartite,
-			      int lasttoggle_flag);
+			      int lasttoggle_flag, int time, int *lasttoggle);
 void WtNetworkDestroy(WtNetwork *nwp);
+WtNetwork WtNetworkInitializeD(double *tails, double *heads, double *weights, Edge nedges,
+			       Vertex nnodes, int directed_flag, Vertex bipartite,
+			      int lasttoggle_flag, int time, int *lasttoggle);
+
+WtNetwork *WtNetworkCopy(WtNetwork *dest, WtNetwork *src);
 
 /* Accessors. */
 Edge WtEdgetreeSearch (Vertex a, Vertex b, WtTreeNode *edges);
+double WtGetEdge (Vertex tail, Vertex head, WtNetwork *nwp);
 Edge WtEdgetreeSuccessor (WtTreeNode *edges, Edge x);
+Edge WtEdgetreePredecessor (WtTreeNode *edges, Edge x);
 Edge WtEdgetreeMinimum (WtTreeNode *edges, Edge x);
+Edge WtEdgetreeMaximum (WtTreeNode *edges, Edge x);
 
 /* Modifiers. */
 
-/* *** don't forget, tail -> head, so these functions now accept tails first, rather than heads */
+/* *** don't forget,  tails -> heads, so all the functions below using
+   heads & tails, now list tails before heads */
 
 
+void WtSetEdge (Vertex tail, Vertex head, double weight, WtNetwork *nwp);
+void WtSetEdgeWithTimestamp (Vertex tail, Vertex head, double weight, WtNetwork *nwp);
 int WtToggleEdge (Vertex tail, Vertex head, double weight, WtNetwork *nwp);
 int WtToggleEdgeWithTimestamp (Vertex tail, Vertex head, double weight, WtNetwork *nwp);
 int WtAddEdgeToTrees(Vertex tail, Vertex head, double weight, WtNetwork *nwp);
@@ -99,7 +108,8 @@ int WtElapsedTime (Vertex tail, Vertex head, WtNetwork *nwp);
 void WtTouchEdge(Vertex tail, Vertex head, WtNetwork *nwp);
 
 /* Utility functions. */
-int WtFindithEdge (Vertex *tail, Vertex *head, Edge i, WtNetwork *nwp);
+int WtFindithEdge (Vertex *tail, Vertex *head, double *weight, Edge i, WtNetwork *nwp);
+int WtGetRandEdge(Vertex *tail, Vertex *head, double *weight, WtNetwork *nwp);
 void Wtprintedge(Edge e, WtTreeNode *edges);
 void WtInOrderTreeWalk(WtTreeNode *edges, Edge x);
 void WtNetworkEdgeList(WtNetwork *nwp);
@@ -107,17 +117,6 @@ void WtShuffleEdges(Vertex *tails, Vertex *heads, double *weights, Edge nedges);
 
 /* Others... */
 Edge WtDesignMissing (Vertex a, Vertex b, WtNetwork *mnwp);
-Edge EdgeTree2EdgeList(Vertex *tails, Vertex *heads, Network *nwp, Edge nmax);
+Edge WtEdgeTree2EdgeList(Vertex *tails, Vertex *heads, double *weights, WtNetwork *nwp, Edge nmax);
 
-/* Below are some functions that only exist for weighted (valued) networks */
-double EdgeWeight (Vertex tail, Vertex head, WtNetwork *nwp);
 #endif
-
-
-
-
-
-
-
-
-
